/*
 * IR transmitter
 *
 * Belongs to:
 * "Mastering Microcontrollers Helped by Arduino"
 * ISBN 978-1-907920-23-3 (English)
 * ISBN 978-2-86661-190-3 (French)
 * http://www.polyvalens.com/
 *
 * For use with PolyValens Multipurpose Shield 129009-1
 *
 * Copyright (c) 2014, Clemens Valens
 *
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR
 * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

#define CARRIER  38000
#define RC_CODE  0x6170807F

int ir_led = 13; // Pin 11 is also an option.
uint16_t startbit = 9000;
uint16_t on_time_0 = 560;
uint16_t off_time_0 = 565;
uint16_t on_time_1 = 560;
uint16_t off_time_1 = 1690;
extern volatile long timer2_toggle_count;

void ir_send_pulse(uint8_t pin, uint32_t frequency, uint32_t duration)
{
  // Overflow bug in Arduino, we have to calculate our own toggle_count.
  // A fudge factor is needed to correct for storing the value
  // in timer2_toggle_count. Carrier frequency dependent.
  unsigned long toggle_count = 2*frequency*duration/1000000 - 8; // 6 for 36 kHz;
  //TCNT2 = 0; // Put this in tone.cpp right before writing OCR2A (line 386 in arduino 1.0.5)
  tone(pin,frequency,duration);
  timer2_toggle_count = toggle_count;
}

void ir_send_bit(uint8_t pin, uint16_t frequency, uint16_t on_time, uint16_t off_time)
{
  ir_send_pulse(pin,frequency,on_time);
  // Wait for the burst to end.
  // OCR2A is cleared in disableTimer at the end of the burst.
  while (OCR2A!=0); 
  // About 104 us are needed to compensate for the delay between
  // the call of ir_send_pulse and the start of the burst.
  delayMicroseconds(off_time-104);
}

void ir_send_code(uint8_t pin, uint16_t frequency, uint32_t code)
{
  ir_send_bit(pin,frequency,startbit,startbit/2);
  for (uint32_t mask=0x80000000; mask!=0; mask>>=1)
  {
    if ((code&mask)==0)
    {
      ir_send_bit(pin,frequency,on_time_0,off_time_0);
    }
    else
    {
      ir_send_bit(pin,frequency,on_time_1,off_time_1);
    }
  }
  ir_send_bit(pin,frequency,on_time_1,off_time_1);
}

void setup(void)
{
  ir_send_code(ir_led,CARRIER,RC_CODE);
}

void loop(void)
{
  // Stop timer0 during transmission.
  bitClear(TIMSK0,TOIE0);
  ir_send_code(ir_led,CARRIER,RC_CODE);
  bitSet(TIMSK0,TOIE0);
  delay(1000);
}
